@@ -81,6 +81,7 @@ _RESOURCE_MULTIPLE_ATTR_ID_MAP = {
 class _Resource(object):
     """Implementation of Redmine resource"""
     redmine_version = None
+    requirements = ()
     container_all = None
     container_one = None
     container_filter = None
@@ -172,9 +173,9 @@ class _Resource(object):
         if item in self._members or item.startswith('_'):
             super(_Resource, self).__setattr__(item, value)
         elif item in self._create_readonly and self.is_new():
-            raise ReadonlyAttrError()
+            raise ReadonlyAttrError
         elif item in self._update_readonly and not self.is_new():
-            raise ReadonlyAttrError()
+            raise ReadonlyAttrError
         elif item == 'custom_fields':
             for org_index, org_field in enumerate(self._attributes.setdefault('custom_fields', [])):
                 if 'value' not in org_field:
@@ -183,9 +184,10 @@ class _Resource(object):
                 try:
                     for new_index, new_field in enumerate(value):
                         if org_field['id'] == new_field['id']:
-                            self._attributes['custom_fields'][org_index]['value'] = value.pop(new_index)['value']
+                            self._attributes['custom_fields'][org_index]['value'] = self.manager.prepare_params(
+                                value.pop(new_index))['value']
                 except (TypeError, KeyError):
-                    raise CustomFieldValueError()
+                    raise CustomFieldValueError
 
             self._attributes['custom_fields'].extend(value)
             self._changes[item] = self._attributes['custom_fields']
@@ -265,9 +267,9 @@ class _Resource(object):
         raise_attr_exception = self.manager.redmine.raise_attr_exception
 
         if isinstance(raise_attr_exception, bool) and raise_attr_exception:
-            raise ResourceAttrError()
+            raise ResourceAttrError
         elif isinstance(raise_attr_exception, (list, tuple)) and self.__class__.__name__ in raise_attr_exception:
-            raise ResourceAttrError()
+            raise ResourceAttrError
 
         return None
 
@@ -347,7 +349,7 @@ class Issue(_Resource):
             self._issue_id = issue.internal_id
 
             if self._redmine.ver is not None and LooseVersion(str(self._redmine.ver)) < LooseVersion('2.3'):
-                raise ResourceVersionMismatchError()
+                raise ResourceVersionMismatchError
 
         def add(self, user_id):
             """Adds user to issue watchers list"""
@@ -371,9 +373,9 @@ class Issue(_Resource):
 
     def __setattr__(self, item, value):
         if item == 'version_id':
-            return super(Issue, self).__setattr__('fixed_version_id', value)
-
-        return super(Issue, self).__setattr__(item, value)
+            super(Issue, self).__setattr__('fixed_version_id', value)
+        else:
+            super(Issue, self).__setattr__(item, value)
 
     def __str__(self):
         try:
@@ -447,6 +449,9 @@ class Attachment(_Resource):
     container_one = 'attachment'
     query_one = '/attachments/{0}.json'
 
+    def download(self, savepath=None, filename=None):
+        return self.manager.redmine.download(self.content_url, savepath, filename)
+
     def __str__(self):
         try:
             return to_string(self.filename)
@@ -517,7 +522,7 @@ class WikiPage(_Resource):
 
     @property
     def internal_id(self):
-        return self.title
+        return to_string(self.title)
 
     def __getattr__(self, item):
         # If a text attribute of a resource is missing, we should
@@ -534,13 +539,13 @@ class WikiPage(_Resource):
         return self.version
 
     def __str__(self):
-        return to_string(self.title)
+        return self.internal_id
 
     def __repr__(self):
         return '<{0}.{1} "{2}">'.format(
             self.__class__.__module__,
             self.__class__.__name__,
-            to_string(self.title)
+            self.internal_id
         )
 
 
